;+
; NAME : disk_despike.pro (procedure)
; PURPOSE :
;    fill spikes using median values of the surrounding pixels
; PROJECT :
;   ASO-S / LST
; CATEGORY :
;   data processing / data analysis
; CALLING SEQUENCE :
;   disk_despike,p1,p2,p3,p4,la_cosmic=la_cosmic
; INPUTS :
;   hdr_in = the file header of img_in (data array)
;   img_in = image to be fixed (fitsfile or a data array with 2 dimensions)
; OPTIONAL INPUT PARAMETERS :
;   None
; OUTPUTS :
;   hdr_out = the file header updated
;   img_out = the image fixed
; OPTIONAL OUTPUT PARAMETERS :
;   None
; KEYWORD PARAMETERS :
;   la_cosmic= if set, la_cosmic.pro is used to search for the spike pixels
;   wst= if set, wst_ind_spike.pro is used to search for the spike pixels; 
;   default, sdi_ind_spike.pro is used to search for the spike pixels 
; COMMON BLOCKS :
;   None
; SIDE EFFECTS :
;   None
; RESTRICTIONS :
;   The input image should be a fitsfile or a 2D data array
; PROCEDURES :
;   read_lst, sdi_ind_spike, wst_ind_spike
; EXAMPLES :
;   disk_despike,hdr1,img1,hdr15,img15
; MODIFICATION HISTORY :
;   2022/03/11 PingZhang
;   2022/04/11 PingZhang
;   2023/04/09 PingZhang
;------------------------------------------------------------------------
;-
;
pro disk_despike,p1,p2,p3,p4,la_cosmic=la_cosmic, wst=wst
  if n_params() eq 3 then begin
    read_lst,p1,hdr_in,img_in,/struct
  endif else begin
    img_in=p2
    hdr_in=p1
  endelse
  
  ;if (datatype(hdr_in) ne 'STR') then hdr_in=struct2fitshead(hdr_in)
  
    if not keyword_set(la_cosmic) then begin
      if not keyword_set(wst) then begin
        sdi_ind_spike,hdr_in,img_in,spkPos=sp,spkValue=spkValue,nSpks=nSpks
        ;sp=spkPos
       if sp eq !NULL then sp=-1
      endif else begin
        wst_ind_spike,hdr_in,img_in,spkPos=sp,spkValue=spkValue,nSpks=nSpks
        ;sp=spkPos
      if sp eq !NULL then sp=-1
      endelse
    endif else begin
      if not keyword_set(wst) then begin
        sdi_ind_spike,hdr_in,img_in,spkPos=spkPos,spkValue=spkValue,nSpks=nSpks,niter=1,/la_cosmic ;for default, niter=4
        sp=spkPos
      if sp eq !NULL then sp=-1
      endif else begin
        wst_ind_spike,hdr_in,img_in,spkPos=spkPos,spkValue=spkValue,nSpks=nSpks,niter=1,/la_cosmic ;for default, niter=4
        sp=spkPos
      if sp eq !NULL then sp=-1
      endelse
    endelse
  
  if (sp[0] ne -1) then begin
    data=img_in
    data(sp)=!VALUES.F_NAN
    ds=size(data)
    nx=ds[1]
    ny=ds[2]
    DATA2 = MAKE_ARRAY(NX+4,NY+4,TYPE=TP)
    DATA2[2,2]=DATA
    W_SPIKE=WHERE((FINITE(DATA2) NE 1), COUNT, COMPLEMENT=W_GOOD, NCOMPLEMENT=COUNT2)
    DATA2[0,*]=DATA2[4,*]
    DATA2[1,*]=DATA2[3,*]
    DATA2[NX+3,*]=DATA2[NX-1,*]
    DATA2[NX+2,*]=DATA2[NX,*]
    DATA2[*,0]=DATA2[*,4]
    DATA2[*,1]=DATA2[*,3]
    DATA2[*,NY+3]=DATA2[*,NY-1]
    DATA2[*,NY+2]=DATA2[*,NY]
    ; FILL IN LABELED SPIKE PIXELS
    NXN=NX+4
    SRRD=[-2*NXN-2,-2*NXN-1,-2*NXN,-2*NXN+1,-2*NXN+2,-NXN-2,-NXN+2,-2,2,NXN-2,NXN+2,2*NXN-2,2*NXN-1,2*NXN,2*NXN+1,2*NXN+2]
    IMOUT=DATA2
    IMOUT[2,2]=DATA
    IF W_SPIKE[0] GE 0 THEN FOR I=0L,N_ELEMENTS(W_SPIKE)-1 DO $
      IMOUT(W_SPIKE(I))=MEDIAN(IMOUT(W_SPIKE(I)+SRRD))
    IMG_OUT=IMOUT[2:NX+1,2:NY+1]
  endif else begin
    img_out=img_in
  endelse
  ;lst_gen_hdr_cal,hdr_in,img_out,outputHdr=outputHdr 
  hdr_in.NSPIKES=0
  ;fxaddpar,outputHdr, 'NSPIKES',0,'the number of spike pixels in the image'
  hdr_out=hdr_in
  ;if (datatype(hdr_out) ne 'STR') then hdr_out=struct2fitshead(hdr_out)
  if n_params() eq 3 then begin
    p2=hdr_out
    p3=img_out
  endif else begin
    p3=hdr_out
    p4=img_out
  endelse
end
