;+
; NAME : lst_lct.pro
;
; PURPOSE : load 'standard' color tables for LST images
;
; PROJECT :
;   ASO-S / LST
; CATEGORY :
;
; CALLING SEQUENCE : lst_lct, wavelnth = wavelnth, load = load 
; 
; INPUTS : wavelnth  in Angstrom 
; 
; OPTIONAL INPUT PARAMETERS : None
;
; OUTPUTS : None
;
; OPTIONAL OUTPUT PARAMETERS : None
;
; KEYWORD PARAMETERS : instr, only need to be set (instr = 'sdi' or instr ='sciuv') when the wavelnth equals 1216 
;
; COMMON BLOCKS : None
; 
; SIDE EFFECTS : None
;
; RESTRICTIONS : None
;
; PROCEDURE :
;
; EXAMPLE :  lst_lct, wavelnth = 1216, instr = 'sdi'
;            lst_lct, wavelnth = 3600
; MODIFICATION HISTORY : witten by Jie Zhao on 2020/03/26
;                        load selected colortable on 2022/01/20
;                         
;------------------------------------------------------------------------
;-
pro lst_lct,wavelnth=wavelnth,instr= instr,load=load 

default,load,1

if not(keyword_set(wavelnth)) then begin   
   print,'lst_lct: must set keyword wavelnth'   
   return    
endif 

if wavelnth eq 1216 then begin
   if not(keyword_set(instr)) then begin   
      print,'lst_lct: must set keyword instr equal sciuv or sdi if wavelnth eq 1216'   
   return
   endif    
endif 

c0=byte(findgen(256)) 
c1=byte(sqrt(findgen(256))*sqrt(255.)) 
c2=byte(findgen(256)^2/255.) 
c3=byte((c1+c2/2.)*255./(max(c1)+max(c2)/2.)) 

loadct,1,/silent 
tvlct,r0,g0,b0,/get
r_uvc = r0
g_uvc = g0
b_uvc = c0


loadct,1,/silent 
gamma_ct,0.6,/current
tvlct,r0,g0,b0,/get
r_uvd = r0
g_uvd = g0
b_uvd = b0

loadct,9,/silent 
tvlct,r0,g0,b0,/get
r_wlc = r0
g_wlc = g0
b_wlc = c2

loadct,0,/silent
tvlct,r0,g0,b0,/get
r_wld = r0
g_wld = g0
b_wld = b0


; allowed values of wavelnth: 
wave=[1216,3600,7000] 
select=where(nint(wavelnth) eq wave) 
if select(0) eq -1 then begin
  print,'lst_lct: selected invalid wavelength/channel'  
  return 
endif

case select of 
0: begin  ; 1216
   if strlowcase(instr) eq 'sciuv' then r=r_uvc & g=g_uvc & b=b_uvc
   if strlowcase(instr) eq 'sdi' then r=r_uvd & g=g_uvd & b=b_uvd
   end
1: begin  ; 3600 
   r=r_wld   
   g=g_wld   
   b=b_wld   
   end    
2: begin  ; 7000
   r=r_wlc
   g=g_wlc
   b=b_wlc
   end
endcase

if load eq 1 then tvlct,r,g,b

end
