function lst_update_history,hdr,str,value=value,history=history,comment=comment

;+
; NAME : 
;   lst_update_history (function)
; PURPOSE :
;   Update the 'history' or 'comment' tag of the fits header in data production
; PROJECT :
;   ASO-S / LST
; CATEGORY :
;   Data processing
; CALLING SEQUENCE :
;   hdr = lst_update_history(hdr,str,value=value)
;   hdr = lst_update_history(hdr,str,value=value,/history,/comment)
; INPUTS :
;   hdr - fits header (string or struct type)
;   str - string that needs to be added to the 'history' or 'comment' tag of the fits header
; OPTIONAL INPUT PARAMETERS :
;   value - a non string varible to be added to 'history' or 'comment' tag
;   history - specify the string to add to the 'history' tag (Default)
;   comment - specify the string to add to the 'comment' tag
; OUTPUTS :
;   hdr - updated header
; OPTIONAL OUTPUT PARAMETERS :
;   None
; KEYWORD PARAMETERS :
;   None
; COMMON BLOCKS :   
;   None
; SIDE EFFECTS :
;   None
; RESTRICTIONS :
;   None
; PROCEDURE CALLED:
;   None
; EXAMPLE :
;   hdr = lst_update_history(hdr, 'LST', value=2022)
; MODIFICATION HISTORY :
;   2022/06/09 Written by Guanglu Shi (shigl@pmo.ac.cn) 

;--Check Header-------------------------------------------------------
IF (DATATYPE(hdr) NE 'STC') THEN hdr = lst_fitshead2struct(hdr, ncomment=20, nhistory=20)

if ~keyword_set(history) and ~keyword_set(comment) then history=1

IF keyword_set(history) THEN BEGIN

  IF tag_exist(hdr, 'HISTORY') THEN BEGIN

    ind = where(hdr.history eq '', num)
    pos = ind[0]
    ; pos = n_elements(hdr[0].history) - num

  ENDIF ELSE BEGIN
  
    message, 'NO HISTORY ARRAY IN HEADER STRUCTURE; hdr unchanged',/inform
    RETURN,hdr

  ENDELSE
  
  ;--Check if history is full
  IF pos EQ -1 THEN BEGIN 
  
    message, 'HEADER STRUCTURE HISTORY ARRAY FULL; hdr unchanged',/inform
    RETURN,hdr

  ENDIF

  ;--Change value type
  IF keyword_set(value) THEN str_value=strtrim(string(value),2) ELSE str_value=''

  hdr.history[pos]=str + ' '+str_value

endif

IF keyword_set(comment) THEN BEGIN

  IF tag_exist(hdr,'COMMENT') THEN BEGIN

    ind=where(hdr.comment eq '',num)
    pos = ind[0]
    ; pos=n_elements(hdr[0].comment)-num

  ENDIF ELSE BEGIN
  
    message, 'NO COMMENT ARRAY IN HEADER STRUCTURE; hdr unchanged',/inform
    RETURN,hdr

  ENDELSE
  
  ;--Check if comment is full
  IF pos EQ -1 THEN BEGIN 
  
    message, 'HEADER STRUCTURE COMMENT ARRAY FULL; hdr unchanged',/inform
    RETURN,hdr

  ENDIF

  ;--Change value type
  IF keyword_set(value) THEN str_value=strtrim(string(value),2) ELSE str_value=''

  hdr.comment[pos]=str + ' '+str_value

endif

RETURN,hdr

END
