;+
; NAME : SDI_FIX_MISSING.PRO (PROCEDURE)
; PURPOSE :
;   FIX BAD OR MISSING PIXELS OF SDI IMAGE AND UPDATE THE STATISTIC IN FILE HEADER.
; PROJECT :
;   ASO-S / LST
; CATEGORY :
;   DATA PROCESSING / DATA ANALYSIS
; CALLING SEQUENCE :
;   SDI_FIX_MISSING,P1,P2,P3,P4,IMG_N=IMG_N,IMG_P=IMG_P
; INPUTS :
;   HDR_IN = THE FILE HEADER OF IMG_IN (DATA ARRAY)
;   IMG_IN = IMAGE TO BE FIXED (FITSFILE OR A DATA ARRAY WITH 2 DIMENSIONS)
; OPTIONAL INPUT PARAMETERS :
;   IMG_N = THE IMAGE OBSERVED IMMEDIATELY BEFORE IMG_IN
;   IMG_P = THE IMAGE OBSERVED IMMEDIATELY AFTER IMG_IN
; OUTPUTS :
;   IMG_OUT = THE IMAGE FIXED
;   HDR_OUT = THE HEADER UPDATED
; OPTIONAL OUTPUT PARAMETERS :
;   NONE
; KEYWORD PARAMETERS :
;   NONE
; COMMON BLOCKS :
;   NONE
; SIDE EFFECTS :
;   NONE
; RESTRICTIONS :
;   THE INPUT IMAGE SHOULD BE A LEVEL 1 FITSFILE OR A 2D DATA ARRAY
; PROCEDURES :
;   read_lst
; EXAMPLES :
;   SDI_FIX_MISSING,(HDR1,IMG1)/FITSFILE_LV1,HDR15,IMG15
; MODIFICATION HISTORY :
;   2021/03/28 PING ZHANG 
;   2021/04/06 PING ZHANG 
;   2022/04/06 PING ZHANG 
;   2022/08/22 PING ZHANG - change missvals, datavals and percentd in fits header

PRO SDI_FIX_MISSING,P1,P2,P3,P4,IMG_N=IMG_N,IMG_P=IMG_P      
  IF N_PARAMS() LT 3 THEN BEGIN
    MESSAGE,'IDL>SDI_FIX_MISSING,P1,P2,P3,P4,IMG_N=IMG_N,IMG_P=IMG_P'
  ENDIF
  IF N_PARAMS() EQ 3 THEN BEGIN
    read_lst,p1,hdr_in,img_in,/struct
  ENDIF ELSE BEGIN
    IMG_IN=P2
    HDR_IN=P1
  ENDELSE
  ;IF (DATATYPE(HDR_IN) NE 'STR') THEN HDR_IN=STRUCT2FITSHEAD(HDR_IN) 
  
  DN=0&DP=0
  IF N_ELEMENTS(IMG_N) NE 0 THEN BEGIN
    IF(DATATYPE(IMG_N) EQ 'STR') THEN BEGIN
      ;DATA_N=READFITS(IMG_N,HEADER)&DN=1
      read_lst,IMG_N,header,data_n
      DN=1
    ENDIF ELSE BEGIN 
    DATA_N=IMG_N&DN=1
  ENDELSE
ENDIF
IF N_ELEMENTS(IMG_P) NE 0 THEN BEGIN
  IF(DATATYPE(IMG_P) EQ 'STR') THEN BEGIN
    DATA_P=READFITS(IMG_P,HEADER)&DP=1
      read_lst,IMG_P,header,data_p
      DP=1
  ENDIF ELSE BEGIN 
  DATA_P=IMG_P&DP=1
ENDELSE
ENDIF
    
  ;T_BEGIN = SYSTIME(/SECONDS)
  TYPE=SIZE(IMG_IN,/TYPE)
  DS=SIZE(IMG_IN)
  NX=DS[1]
  NY=DS[2]

  IF DN+DP EQ 0 THEN BEGIN
    IMG_IN(WHERE(img_in EQ -32768))=!VALUES.F_NAN
    IMG_IN(WHERE(img_in EQ -32767))=!VALUES.F_NAN
    DATA=IMG_IN
    DATA2 = MAKE_ARRAY(NX+4,NY+4,TYPE=TP)
    DATA2[2,2]=DATA
    W_MISSING=WHERE((FINITE(DATA2) NE 1), COUNT, COMPLEMENT=W_GOOD, NCOMPLEMENT=COUNT2)
    DATA2[0,*]=DATA2[4,*]
    DATA2[1,*]=DATA2[3,*]
    DATA2[NX+3,*]=DATA2[NX-1,*]
    DATA2[NX+2,*]=DATA2[NX,*]
    DATA2[*,0]=DATA2[*,4]
    DATA2[*,1]=DATA2[*,3]
    DATA2[*,NY+3]=DATA2[*,NY-1]
    DATA2[*,NY+2]=DATA2[*,NY]
    ; FILL IN LABELED MISSING PIXELS
    NXN=NX+4
    SRRD=[-2*NXN-2,-2*NXN-1,-2*NXN,-2*NXN+1,-2*NXN+2,-NXN-2,-NXN+2,-2,2,NXN-2,NXN+2,2*NXN-2,2*NXN-1,2*NXN,2*NXN+1,2*NXN+2]
    IMOUT=DATA2
    IMOUT[2,2]=DATA
    IF W_MISSING[0] GE 0 THEN FOR I=0L,N_ELEMENTS(W_MISSING)-1 DO $
      IMOUT(W_MISSING(I))=MEDIAN(IMOUT(W_MISSING(I)+SRRD))
      
    IMG_OUT=IMOUT[2:NX+1,2:NY+1]
    IMG_OUT(WHERE(FINITE(IMG_OUT) NE 1,COUNTM))=-32768
    LST_GEN_HDR_CAL,HDR_IN,IMG_OUT,OUTPUTHDR=OUTPUTHDR
    totvals=outputHdr.TOTVALS;FXPAR(outputHdr,'TOTVALS')
    outputHdr.MISSVALS=COUNTM;fxaddpar,outputHdr,'MISSVALS', wcnt, 'Number of missing data values'
    outputHdr.DATAVALS=totvals-COUNTM;fxaddpar,outputHdr,'DATAVALS',totvals-wcnt,'Actual number of data values in image (pixels)'
    outputHdr.PERCENTD=round(float(totvals-COUNTM)/float(totvals)*100);fxaddpar,outputHdr,'PERCENTD',float(totvals-wcnt)/float(wcnt),'Percentage of actual/total image data number'
    HDR_OUT=OUTPUTHDR
    if (datatype(hdr_out) ne 'STR') then hdr_out=struct2fitshead(hdr_out)
    IF N_PARAMS() EQ 3 THEN BEGIN
      P2=HDR_OUT
      P3=IMG_OUT
    ENDIF ELSE BEGIN
      P3=HDR_OUT
      P4=IMG_OUT
    ENDELSE
  ENDIF ELSE BEGIN
  
   IF DN EQ 1 AND DP EQ 0 THEN BEGIN
     DATA = MAKE_ARRAY(NX,NY,2,TYPE=TP)
     DATA[*,*,0]=IMG_IN
     DATA[*,*,1]=DATA_N
   ENDIF
   IF DN EQ 0 AND DP EQ 1 THEN BEGIN
     DATA = MAKE_ARRAY(NX,NY,2,TYPE=TP)
     DATA[*,*,0]=IMG_IN
     DATA[*,*,1]=DATA_P
   ENDIF
   IF DN EQ 1 AND DP EQ 1 THEN BEGIN
     DATA = MAKE_ARRAY(NX,NY,3,TYPE=TP)
     DATA[*,*,0]=DATA_N
     DATA[*,*,1]=IMG_IN
     DATA[*,*,2]=DATA_P
   ENDIF
   
   DS=SIZE(DATA)
   IF (DS[0] EQ 3) THEN BEGIN
    DATA(WHERE(DATA EQ -32768))=!VALUES.F_NAN
    DATA(WHERE(DATA EQ -32767))=!VALUES.F_NAN
     N1=PRODUCT(DS[1:2])
     N2=DS(3)
     DATAC=REFORM(DATA,N1,N2,1)
     
     FOR I=0,N1-1 DO BEGIN
       AA = REFORM(DATAC(I,*,0))
       W_MISSING = WHERE((FINITE(AA) NE 1), COUNT, COMPLEMENT=W_GOOD, NCOMPLEMENT=COUNT2)
       IF (COUNT EQ 0) OR (COUNT EQ N_ELEMENTS(AA)) THEN GOTO, SKIP1
       IF COUNT2 EQ 1 THEN BEGIN DATAC(I,*,0) = AA(W_GOOD(0))
     ENDIF ELSE BEGIN
     
       ; FILL IN LABELED MISSING PIXELS
       FOR II=0, COUNT-1 DO BEGIN
         W=W_MISSING[II]
         IF W EQ 0 THEN  W1=W_GOOD(0)&W2=W_GOOD(0)
         IF W EQ N2-1 THEN  W1=W_GOOD(COUNT2-1)&W2=W_GOOD(COUNT2-1)
         IF NOT( W EQ 0 OR W EQ N2-1) THEN BEGIN
           W1 = MAX(W_GOOD(WHERE(W_GOOD LT W)))
           W2 = MIN(W_GOOD(WHERE(W_GOOD GT W)))
           I1 = W1 + 1
           I2 = W2 - 1
           X = LINDGEN(I2-I1+1) + I1
           IF W1 EQ W2 THEN AA(X) = AA(W1) ELSE AA(X)=(AA(W2)*(X-W1)-AA(W1)*(X-W2))/FLOAT(W2-W1)
         ENDIF
       ENDFOR
       DATAC(I,*,0) = AA
     ENDELSE
     SKIP1:
   ENDFOR
   DATA = REFORM(DATAC, DS(1:DS(0)))
   IF DN+DP EQ 1 THEN IMG_OUT=DATA(*,*,0)
   IF DN+DP EQ 2 THEN IMG_OUT=DATA(*,*,1)
   IMG_OUT(WHERE(FINITE(IMG_OUT) NE 1,COUNTM))=-32768
   ;LST_GEN_HDR_CAL,HDR_IN,IMG_OUT,OUTPUTHDR=OUTPUTHDR
    totvals=hdr_in.TOTVALS;FXPAR(outputHdr,'TOTVALS')
    hdr_in.MISSVALS=COUNTM;fxaddpar,outputHdr,'MISSVALS', wcnt, 'Number of missing data values'
    hdr_in.DATAVALS=totvals-COUNTM;fxaddpar,outputHdr,'DATAVALS',totvals-wcnt,'Actual number of data values in image (pixels)'
    hdr_in.PERCENTD=float(totvals-COUNTM)/float(totvals)*100;'Percentage of actual/total image data number'
   HDR_OUT=hdr_in
   ;if (datatype(hdr_out) ne 'STR') then hdr_out=struct2fitshead(hdr_out)
   IF N_PARAMS() EQ 3 THEN BEGIN
     P2=HDR_OUT
     P3=IMG_OUT
   ENDIF ELSE BEGIN
     P3=HDR_OUT
     P4=IMG_OUT
   ENDELSE
 ENDIF
ENDELSE
  ;T_END = SYSTIME(/SECONDS)
  ;PRINT,SYSTIME()+' SDI_FIX_MISSING FINISHED, ',T_END-T_BEGIN,' SEC ELAPSED.'
END
